#include "Data_Analysis.h"

#include <iostream>

using namespace Forecast_Util;

double Data_Analysis::K_Mean(Param_List& pm, Field_Data_Manager& dm)
{
    
    GVector* p_sf1_a = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER1_AFTER);
    GVector* p_sf1_b = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER1_BEFORE);
    GVector* p_sf3_a = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER3_AFTER);
    GVector* p_sf3_b = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER3_BEFORE);
    
    double J = p_sf1_a->Size() - 1;
    double conf = 10.0 / ( 2.0 * pm.del_y * pm.del_y );
    double tmp = 0.0;
    double sf1_m_jup, sf1_m_j, sf3_m_jup, sf3_m_j;

    for(int j = 1; j < J; j++){
        sf1_m_j = time_ave_value(p_sf1_a, p_sf1_b, j);
        sf1_m_jup = time_ave_value(p_sf1_a, p_sf1_b, j+1);
        sf3_m_j = time_ave_value(p_sf3_a, p_sf3_b, j);
        sf3_m_jup = time_ave_value(p_sf3_a, p_sf3_b, j+1);

	tmp += ( sf1_m_jup - sf1_m_j ) * ( sf1_m_jup - sf1_m_j ) 
	       + ( sf3_m_jup - sf3_m_j ) * ( sf3_m_jup - sf3_m_j );
    }

    return conf * tmp / J;
}

double Data_Analysis::K_Disturb(Param_List& pm, Field_Data_Manager& dm)
{
    GMatrix* p_sf1_a = dm.Get_Stream_Func_Disturb(Field_Data_Manager::SF_DISTB_LAYER1_AFTER);
    GMatrix* p_sf1_b = dm.Get_Stream_Func_Disturb(Field_Data_Manager::SF_DISTB_LAYER1_BEFORE);
    GMatrix* p_sf3_a = dm.Get_Stream_Func_Disturb(Field_Data_Manager::SF_DISTB_LAYER3_AFTER);
    GMatrix* p_sf3_b = dm.Get_Stream_Func_Disturb(Field_Data_Manager::SF_DISTB_LAYER3_BEFORE);
    
    double J = p_sf1_a->Column() - 1;
    double I = p_sf1_a->Row();
    double conf = 10.0 / ( 2.0 * J * pm.del_x * pm.del_x );
    double tmp = 0.0;
    double tmp1, tmp2;
    double ep2 = ( pm.del_x * pm.del_x ) / ( pm.del_y * pm.del_y );
    double sf1_ijup, sf1_iupj, sf1_ij;
    double sf3_ijup, sf3_iupj, sf3_ij;

    for(int i=0; i < I; i++){
        for(int j = 0; j < J; j++){
            sf1_ij = time_ave_value(p_sf1_a, p_sf1_b, i, j);
            sf1_iupj = time_ave_value(p_sf1_a, p_sf1_b, i+1, j);
            sf1_ijup = time_ave_value(p_sf1_a, p_sf1_b, i, j+1);

            sf3_ij = time_ave_value(p_sf3_a, p_sf3_b, i, j);
            sf3_iupj = time_ave_value(p_sf3_a, p_sf3_b, i+1, j);
            sf3_ijup = time_ave_value(p_sf3_a, p_sf3_b, i, j+1);
	    
	    tmp += ( sf1_iupj - sf1_ij ) * ( sf1_iupj - sf1_ij )
	           + ( sf3_iupj - sf3_ij ) * ( sf3_iupj - sf3_ij )
		   + 
		   ep2 * 
		    ( 
	             ( sf1_ijup - sf1_ij ) * ( sf1_ijup - sf1_ij )
	             + ( sf3_ijup - sf3_ij ) * ( sf3_ijup - sf3_ij )
		    );
	}
    }

    return conf * tmp / I;
}
    
double Data_Analysis::P_Mean(Param_List& pm, Field_Data_Manager& dm)
{
    GVector* p_sf1_a = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER1_AFTER);
    GVector* p_sf1_b = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER1_BEFORE);

    GVector* p_sf3_a = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER3_AFTER);
    GVector* p_sf3_b = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER3_BEFORE);
    
    double J = p_sf1_a->Size() - 1;
    double conf = 10.0 * pm.vertical_stability / 2.0;
    double tmp = 0;
    double sf1_m_j, sf3_m_j;

    for(int j = 1; j < J; j++){
        sf1_m_j = time_ave_value(p_sf1_a, p_sf1_b, j); 
        sf3_m_j = time_ave_value(p_sf3_a, p_sf3_b, j); 

	tmp += ( sf1_m_j - sf3_m_j ) * ( sf1_m_j - sf3_m_j );
    }

    return conf * tmp / J;
}

double Data_Analysis::P_Disturb(Param_List& pm, Field_Data_Manager& dm)
{
    GMatrix* p_sf1_a = dm.Get_Stream_Func_Disturb(Field_Data_Manager::SF_DISTB_LAYER1_AFTER);
    GMatrix* p_sf1_b = dm.Get_Stream_Func_Disturb(Field_Data_Manager::SF_DISTB_LAYER1_BEFORE);
    GMatrix* p_sf3_a = dm.Get_Stream_Func_Disturb(Field_Data_Manager::SF_DISTB_LAYER3_AFTER);
    GMatrix* p_sf3_b = dm.Get_Stream_Func_Disturb(Field_Data_Manager::SF_DISTB_LAYER3_BEFORE);
    
    int I = pm.num_x;
    int J = pm.num_y - 1;
    double tmp = 0.0;
    double conf = 10.0 / ( 2.0 * pm.del_y * pm.del_y );
    double sf1_ij, sf3_ij;

    for(int i=0; i < I; i++){
        for(int j=0; j < J; j++){
            sf1_ij = time_ave_value(p_sf1_a, p_sf1_b, i, j);
            sf3_ij = time_ave_value(p_sf3_a, p_sf3_b, i, j);

            tmp += ( sf1_ij - sf3_ij ) * ( sf1_ij - sf3_ij );
	}
    }

    return conf * tmp / ( I * J );
}

double Data_Analysis::Mean_Q_To_Mean_P(Param_List& pm, Field_Data_Manager& dm)
{
    GVector* p_sf1_m_a = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER1_AFTER);
    GVector* p_sf1_m_b = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER1_BEFORE);

    GVector* p_sf3_m_a = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER3_AFTER);
    GVector* p_sf3_m_b = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER3_BEFORE);
    
    double sf1_m_j, sf3_m_j;
    double jmax = p_sf1_m_a->Size();
    
    double conf = - 2.0 * Earth::R_dry * pm.H * pm.vertical_stability * l 
                  / ( pm.f0 * Earth::Cp_dry );
    double J = jmax - 1;
    double tmp = 0;

    for(int j = 1; j < J; j++){
        sf1_m_j = time_ave_value( p_sf1_m_a, p_sf1_m_b, j);
        sf3_m_j = time_ave_value( p_sf3_m_a, p_sf3_m_b, j);

	tmp += ( sf1_m_j - sf3_m_j ) * ( 2.0 * j - J ) / J;
    }

    return conf * tmp / J;
}

double Data_Analysis::Mean_P_To_Mean_K(Param_List& pm, Field_Data_Manager& dm)
{
    GVector* p_omega_m = dm.Get_Omega_Mean();
    GVector* p_sf1_m_a = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER1_AFTER);
    GVector* p_sf1_m_b = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER1_BEFORE);
    GVector* p_sf3_m_a = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER3_AFTER);
    GVector* p_sf3_m_b = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER3_BEFORE);
    
    double sf1_m_j, sf3_m_j;
    double jmax = p_sf1_m_a->Size();
    
    double conf = - pm.f0 * l / pm.p2;
    double J = jmax - 1;
    double tmp = 0;
    
    for(int j=1; j < J; j++){
        sf1_m_j = time_ave_value( p_sf1_m_a, p_sf1_m_b, j);
        sf3_m_j = time_ave_value( p_sf3_m_a, p_sf3_m_b, j);

	tmp += p_omega_m->Get(j) * ( sf1_m_j - sf3_m_j );
    }

    return conf * tmp / J;
}

double Data_Analysis::Mean_K_To_A(Param_List& pm, Field_Data_Manager& dm)
{
    GVector* p_vr1 = dm.Get_Vorcity_Mean(Field_Data_Manager::VR_LAYER1);
    GVector* p_vr3 = dm.Get_Vorcity_Mean(Field_Data_Manager::VR_LAYER3);
    
    double jmax = p_vr1->Size();
    double J = jmax - 1;
    double tmp = 0;
    double zeta1_j, zeta3_j;
    
    for(int j=1; j < J; j++){
        zeta1_j = p_vr1->Get(j);
        zeta3_j = p_vr3->Get(j); 

	tmp += zeta1_j * zeta1_j + zeta3_j * zeta3_j;
    }

    return pm.A * l * tmp / J;
}

double Data_Analysis::Disturb_K_To_A(Param_List& pm, Field_Data_Manager& dm)
{
    GMatrix* p_vr1 = dm.Get_Vorcity_Disturb(Field_Data_Manager::VR_LAYER1);
    GMatrix* p_vr3 = dm.Get_Vorcity_Disturb(Field_Data_Manager::VR_LAYER3);
    
    double jmax = p_vr1->Column();
    double I = pm.num_x;
    double J = jmax - 1;
    double tmp = 0;
    double zeta1_j, zeta3_j;
    
    for(int i=0; i < I; i++){
        for(int j=1; j < J; j++){
            zeta1_j = p_vr1->Get(i, j);
            zeta3_j = p_vr3->Get(i, j); 

	    tmp += zeta1_j * zeta1_j + zeta3_j * zeta3_j;
        }
    }

    return pm.A * l * tmp / ( I * J );
}

double Data_Analysis::Mean_P_To_A(Param_List& pm, Field_Data_Manager& dm)
{
    GVector* p_sf1_a = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER1_AFTER);
    GVector* p_sf1_b = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER1_BEFORE);
    GVector* p_sf3_a = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER3_AFTER);
    GVector* p_sf3_b = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER3_BEFORE);
    
    double J = p_sf1_a->Size() - 1;
    double conf = pm.vertical_stability * pm.A * l / ( pm.del_y * pm.del_y );
    double tmp = 0.0;
    double tmp1 = 0.0;
    double sf1_m_jup, sf1_m_j, sf3_m_jup, sf3_m_j;

    for(int j = 1; j < J; j++){
        sf1_m_j = time_ave_value(p_sf1_a, p_sf1_b, j);
        sf1_m_jup = time_ave_value(p_sf1_a, p_sf1_b, j+1);
        sf3_m_j = time_ave_value(p_sf3_a, p_sf3_b, j);
        sf3_m_jup = time_ave_value(p_sf3_a, p_sf3_b, j+1);

	tmp1 = ( sf1_m_jup - sf3_m_jup ) - ( sf1_m_j - sf3_m_j ); 
	tmp += tmp1 * tmp1;
    }


    return conf * tmp / J;
}

double Data_Analysis::Mean_K_To_k(Param_List& pm, Field_Data_Manager& dm)
{
    GVector* p_sf3_m_a = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER3_AFTER);
    GVector* p_sf3_m_b = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER3_BEFORE);
    GVector* p_vr1 = dm.Get_Vorcity_Mean(Field_Data_Manager::VR_LAYER1);
    GVector* p_vr3 = dm.Get_Vorcity_Mean(Field_Data_Manager::VR_LAYER3);

    GVector* p_sf3 = new GVector(pm.num_y);
    
    p_sf3->Add(*p_sf3_m_a, *p_sf3_m_b);
    p_sf3->Scalar_Product(0.5);

    double jmax = p_sf3_m_a->Size();
    double J = jmax - 1;
    double tmp = 0;
    double zeta1_j, zeta3_j, zeta4_j;
    
    for(int j=1; j < J; j++){
            zeta1_j = p_vr1->Get(j);
            zeta3_j = p_vr3->Get(j);
        
            zeta4_j = ( 3.0 * zeta3_j - zeta1_j ) / 2.0; 
	    tmp += zeta4_j * p_sf3->Get(j);
        
    }

    delete p_sf3;
    return - pm.k * l * tmp / J;
}


double Data_Analysis::Mean_P_To_Disturb_P(Param_List& pm, Field_Data_Manager& dm)
{
    GVector* p_sf1_m_a = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER1_AFTER);
    GVector* p_sf1_m_b = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER1_BEFORE);
    GVector* p_sf3_m_a = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER3_AFTER);
    GVector* p_sf3_m_b = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER3_BEFORE);
    
    GMatrix* p_sf1_d_a = dm.Get_Stream_Func_Disturb(Field_Data_Manager::SF_DISTB_LAYER1_AFTER);
    GMatrix* p_sf1_d_b = dm.Get_Stream_Func_Disturb(Field_Data_Manager::SF_DISTB_LAYER1_BEFORE);
    GMatrix* p_sf3_d_a = dm.Get_Stream_Func_Disturb(Field_Data_Manager::SF_DISTB_LAYER3_AFTER);
    GMatrix* p_sf3_d_b = dm.Get_Stream_Func_Disturb(Field_Data_Manager::SF_DISTB_LAYER3_BEFORE);
    
    GMatrix* p_sf1_d = new GMatrix(pm.num_x, pm.num_y);
    GMatrix* p_sf3_d = new GMatrix(pm.num_x, pm.num_y);

    p_sf1_d->Add(*p_sf1_d_a, *p_sf1_d_b);
    p_sf1_d->Scalar_Product(0.5);
    p_sf3_d->Add(*p_sf3_d_a, *p_sf3_d_b);
    p_sf3_d->Scalar_Product(0.5);
    
    double jmax = p_sf1_d_a->Column();
    double conf = - pm.vertical_stability * l / ( 4.0 * pm.del_x * pm.del_y );
    double I = p_sf1_d_a->Row();
    double J = jmax - 1;
    double tmp, tmpJ;
    double sf1_m_j, sf3_m_j; 
    
    tmp = 0;
    for(int j=1; j < J; j++){
        tmpJ = 0;

	for(int i=1; i < I; i++){
	    tmpJ += Forecast_Util::J(pm, *p_sf1_d, *p_sf3_d, i, j);
        }
	
        sf1_m_j = time_ave_value( p_sf1_m_a, p_sf1_m_b, j);
        sf3_m_j = time_ave_value( p_sf3_m_a, p_sf3_m_b, j);	
	tmp += ( sf1_m_j - sf3_m_j ) * tmpJ;
    }
    
    delete p_sf1_d;
    delete p_sf3_d;

    return conf * tmp / ( I * J );
}

double Data_Analysis::Disturb_P_To_Disturb_K(Param_List& pm, Field_Data_Manager& dm)
{
    GMatrix* p_omega_d = dm.Get_Omega_Disturb();
    GMatrix* p_sf1_d_a = dm.Get_Stream_Func_Disturb(Field_Data_Manager::SF_DISTB_LAYER1_AFTER);
    GMatrix* p_sf1_d_b = dm.Get_Stream_Func_Disturb(Field_Data_Manager::SF_DISTB_LAYER1_BEFORE);
    GMatrix* p_sf3_d_a = dm.Get_Stream_Func_Disturb(Field_Data_Manager::SF_DISTB_LAYER3_AFTER);
    GMatrix* p_sf3_d_b = dm.Get_Stream_Func_Disturb(Field_Data_Manager::SF_DISTB_LAYER3_BEFORE);
    
    double sf1_d_ij, sf3_d_ij;
    double jmax = p_sf1_d_a->Column();

    double conf = - pm.f0 * l / pm.p2;
    double I = p_sf1_d_a->Row();
    double J = jmax - 1;
    double tmp = 0;
    
    for(int i=0; i < I; i++){
        for(int j=1; j < J; j++){
            sf1_d_ij = time_ave_value( p_sf1_d_a, p_sf1_d_b, i, j);
            sf3_d_ij = time_ave_value( p_sf3_d_a, p_sf3_d_b, i, j);

	    tmp += p_omega_d->Get(i, j) * ( sf1_d_ij - sf3_d_ij );
        }
    }

    return conf * tmp / ( I * J );
}

double Data_Analysis::Disturb_K_To_Mean_K(Param_List& pm, Field_Data_Manager& dm)
{
    GVector* p_sf1_m_a = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER1_AFTER);
    GVector* p_sf1_m_b = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER1_BEFORE);
    GVector* p_sf3_m_a = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER3_AFTER);
    GVector* p_sf3_m_b = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER3_BEFORE);
    
    GMatrix* p_sf1_d_a = dm.Get_Stream_Func_Disturb(Field_Data_Manager::SF_DISTB_LAYER1_AFTER);
    GMatrix* p_sf1_d_b = dm.Get_Stream_Func_Disturb(Field_Data_Manager::SF_DISTB_LAYER1_BEFORE);
    GMatrix* p_sf3_d_a = dm.Get_Stream_Func_Disturb(Field_Data_Manager::SF_DISTB_LAYER3_AFTER);
    GMatrix* p_sf3_d_b = dm.Get_Stream_Func_Disturb(Field_Data_Manager::SF_DISTB_LAYER3_BEFORE);
    
    GMatrix* p_sf1_d = new GMatrix(pm.num_x, pm.num_y);
    GMatrix* p_sf3_d = new GMatrix(pm.num_x, pm.num_y);

    p_sf1_d->Add(*p_sf1_d_a, *p_sf1_d_b);
    p_sf1_d->Scalar_Product(0.5);
    p_sf3_d->Add(*p_sf3_d_a, *p_sf3_d_b);
    p_sf3_d->Scalar_Product(0.5);
    
    double jmax = p_sf1_d_a->Column();
    double conf = l / ( 4.0 * pm.del_x * pm.del_x * pm.del_x * pm.del_y );
    double I = p_sf1_d_a->Row();
    double J = jmax - 1;
    double tmp, tmpL1, tmpL3;
    double sf1_m_jup, sf1_m_jdwn; 
    double sf3_m_jup, sf3_m_jdwn; 
    
    tmp = 0;
    for(int j=1; j < J; j++){
        tmpL1 = 0;
	tmpL3 = 0;

	for(int i=1; i < I; i++){
	    tmpL1 += ( p_sf1_d->Get(i+1, j) - p_sf1_d->Get(i-1, j) )
	              * Forecast_Util::L(pm, *p_sf1_d, i, j);
	    tmpL3 += ( p_sf3_d->Get(i+1, j) - p_sf3_d->Get(i-1, j) )
	              * Forecast_Util::L(pm, *p_sf3_d, i, j);
        }
	
        sf1_m_jup = time_ave_value( p_sf1_m_a, p_sf1_m_b, j+1);
        sf3_m_jup = time_ave_value( p_sf3_m_a, p_sf3_m_b, j+1);	
        sf1_m_jdwn = time_ave_value( p_sf1_m_a, p_sf1_m_b, j-1);
        sf3_m_jdwn = time_ave_value( p_sf3_m_a, p_sf3_m_b, j-1);	
	
	tmp += ( sf1_m_jdwn - sf1_m_jup ) * tmpL1 
	       + ( sf3_m_jdwn - sf3_m_jup ) * tmpL3;
    }
    
    delete p_sf1_d;
    delete p_sf3_d;

    return conf * tmp / ( I * J );
}

double Data_Analysis::Disturb_P_To_A(Param_List& pm, Field_Data_Manager& dm)
{
    GMatrix* p_sf1_a = dm.Get_Stream_Func_Disturb(Field_Data_Manager::SF_DISTB_LAYER1_AFTER);
    GMatrix* p_sf1_b = dm.Get_Stream_Func_Disturb(Field_Data_Manager::SF_DISTB_LAYER1_BEFORE);
    GMatrix* p_sf3_a = dm.Get_Stream_Func_Disturb(Field_Data_Manager::SF_DISTB_LAYER3_AFTER);
    GMatrix* p_sf3_b = dm.Get_Stream_Func_Disturb(Field_Data_Manager::SF_DISTB_LAYER3_BEFORE);
    
    double J = p_sf1_a->Column() - 1;
    double I = p_sf1_a->Row();
    double conf = pm.vertical_stability * pm.A * l / ( pm.del_x * pm.del_x );
    double tmp = 0.0;
    double tmp1, tmp2;
    double ep2 = ( pm.del_x * pm.del_x ) / ( pm.del_y * pm.del_y );
    double sf1_ijup, sf1_iupj, sf1_ij;
    double sf3_ijup, sf3_iupj, sf3_ij;

    for(int i=0; i < I; i++){
        for(int j = 1; j < J; j++){
            sf1_ij = time_ave_value(p_sf1_a, p_sf1_b, i, j);
            sf1_iupj = time_ave_value(p_sf1_a, p_sf1_b, i+1, j);
            sf1_ijup = time_ave_value(p_sf1_a, p_sf1_b, i, j+1);

            sf3_ij = time_ave_value(p_sf3_a, p_sf3_b, i, j);
            sf3_iupj = time_ave_value(p_sf3_a, p_sf3_b, i+1, j);
            sf3_ijup = time_ave_value(p_sf3_a, p_sf3_b, i, j+1);
	    
	    tmp1 = ( sf1_iupj - sf3_iupj ) - ( sf1_ij - sf3_ij ); 
	    tmp2 = ( sf1_ijup - sf3_ijup ) - ( sf1_ij - sf3_ij ); 
	    tmp += tmp1 * tmp1 + ep2 * tmp2 * tmp2;
	}
    }

    return conf * tmp / ( I * J );
}

double Data_Analysis::Disturb_K_To_k(Param_List& pm, Field_Data_Manager& dm)
{
    GMatrix* p_sf3_m_a = dm.Get_Stream_Func_Disturb(Field_Data_Manager::SF_DISTB_LAYER3_AFTER);
    GMatrix* p_sf3_m_b = dm.Get_Stream_Func_Disturb(Field_Data_Manager::SF_DISTB_LAYER3_BEFORE);
    GMatrix* p_vr1_d = dm.Get_Vorcity_Disturb(Field_Data_Manager::VR_LAYER1);
    GMatrix* p_vr3_d = dm.Get_Vorcity_Disturb(Field_Data_Manager::VR_LAYER3);

    GMatrix* p_sf3_d = new GMatrix(pm.num_x, pm.num_y);
    
    p_sf3_d->Add(*p_sf3_m_a, *p_sf3_m_b);
    p_sf3_d->Scalar_Product(0.5);

    double jmax = p_sf3_m_a->Column();
    double J = jmax - 1;
    double I = p_sf3_m_a->Row();
    double tmp = 0;
    double zeta1_ij, zeta3_ij, zeta4_ij;
    
    for(int i=0; i < I; i++){
        for(int j=1; j < J; j++){
            zeta1_ij = p_vr1_d->Get(i, j);
            zeta3_ij = p_vr3_d->Get(i, j);
        
            zeta4_ij = ( 3.0 * zeta3_ij - zeta1_ij ) / 2.0; 
	    tmp += zeta4_ij * p_sf3_d->Get(i, j);
	}
    }

    delete p_sf3_d;
    return - pm.k * l * tmp / ( I * J );
}

