#include "Forecast.h"
#include "Forecast_Util.h"
#include "Data_Analysis.h"

// C++ ̕WCũwb_[t@C̃CN[h
#include <time.h>
#include <stdlib.h>
#include <iostream>

// GMathLib ̃wb_[CN[h
#include <gfd/GConstants.h>


using namespace GMathLib;
using namespace GMathLib::GFD::GConstants;

using namespace Forecast_Util;

Forecast::Forecast(const Param_List& pm)
:time_step(1), distb_cal_flag(true), first_run_flag(false)
{
    // |eVQx̎ԔW\,̉Eӂ̌vZʂ
    // ێIuWFNg쐬.
    p_chi1 = new GMatrix(pm.num_x, pm.num_y);  
    p_chi1_mean = new GVector(pm.num_y);
    p_chi1_distb = new GMatrix(pm.num_x, pm.num_y);

    p_chi3 = new GMatrix(pm.num_x, pm.num_y);
    p_chi3_mean = new GVector(pm.num_y);
    p_chi3_distb = new GMatrix(pm.num_x, pm.num_y);
    
    p_adv_mx1 = new GMatrix(pm.num_x, pm.num_y);  
    p_adv_mx3 = new GMatrix(pm.num_x, pm.num_y);

    p_pv_mean13 = new GVector(pm.num_y);        
    p_pv_distb13 = new GMatrix(pm.num_x, pm.num_y);
    p_A = new GMatrix(pm.num_y, pm.num_y);
    p_A->Fill(0.0);
    
    // ̃NXŗpp[^炩ߌvZĂ.
    Configure_Parameter(pm);

}

Forecast::~Forecast()
{
    if(p_chi1){
        delete p_chi1; delete p_chi1_mean; delete p_chi1_distb;
        delete p_chi3; delete p_chi3_mean; delete p_chi3_distb;

	delete p_adv_mx1; delete p_adv_mx3;
    
	delete p_pv_mean13; delete p_pv_distb13;
	delete p_A;
    }
}

void Forecast::Configure_Parameter(const Param_List& pm)
{
    alpha = pm.A * pm.del_t / ( pm.del_x * pm.del_x );
    b = pm.beta * ( pm.del_x * pm.del_x ) * pm.del_y;  
    c = pm.del_t / ( 2.0 * pm.del_x * pm.del_y );
    h = 4.0 * Earth::R_dry * pm.H * ( pm.vertical_stability * pm.del_x * pm.del_x ) * pm.del_t 
	 / ( pm.f0 * Earth::Cp_dry );
    epsilon = pm.del_x / pm.del_y;
    
    mu1 = 1.0;
    mu3 = 1.0 + 1.5 * pm.k * pm.del_t;
    
    /** vZp[^\. ************************/
    // std::cout << "alpha =" << alpha << std::endl;
    // std::cout << "b = " << b << std::endl;
    // std::cout << "c = " << c << std::endl;
    // std::cout << "h = " << h << std::endl;
    // std::cout << "epsilon = " << epsilon << std::endl; 
    /********************************************************/    
}

void Forecast::Configure_First_Run_Param(Param_List& pm){
     alpha = alpha / 2.0;
     c = c / 2.0;
     h = h / 2.0;
     pm.k = pm.k / 2.0;
     mu3 = 1.0;

}

double Forecast::Prepair(Param_List& pm, Field_Data_Manager& dm)
{
    // ̊֐ɂ,|eV̏ݒ肷.
    GVector* p_sf1 = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER1_AFTER);
    GVector* p_sf3 = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER3_AFTER);
    

    // lƂĐݒ肳Ă闬֐̏ꂩ,|eVQx̏vZĂ.
    double tmp;
    tmp	= - pm.del_x * pm.del_x *  
         pm.vertical_stability * ( p_sf1->Get(0) - p_sf3->Get(0) );
    
    dm.Get_PV(Field_Data_Manager::PV_LAYER1_BEFORE)->Fill(tmp);
    dm.Get_PV(Field_Data_Manager::PV_LAYER1_BEFORE2)->Fill(tmp);
    dm.Get_PV_Mean(Field_Data_Manager::PV_MEAN_LAYER1)->Fill(tmp);

    dm.Get_PV(Field_Data_Manager::PV_LAYER3_BEFORE)->Fill(- tmp);
    dm.Get_PV(Field_Data_Manager::PV_LAYER3_BEFORE2)->Fill(- tmp);
    dm.Get_PV_Mean(Field_Data_Manager::PV_MEAN_LAYER3)->Fill(- tmp);
    
    // first run ̃tO𗧂Ă
    Set_First_Run();
}

void Forecast::Calc_PV_Time_Evol(Param_List& pm, Field_Data_Manager& dm){
     
     // |eVQx̎ԔW̉EӂvZ.     
     if(first_run_flag ){
         // P^CXebvڂ͎ t-1 ͕̕Ȃ̂,OiIC[@ŉ.
	 // ̂߂ɌW𒲐.
         Configure_First_Run_Param(pm);
	 std::cout << "first run" << std::endl;

     }
     
     calc_PV_Time_Evol_Prepair(pm, dm);

     // ߂ p_chi1 ̓ςƂ, p_chi1_mean Ɋi[.
     zonal_average(*p_chi1, *p_chi1_mean);
     // p_chi1, p_chi1_mean gďvZ, p_chi1_distb Ɋi[.
     calc_disturb(*p_chi1, *p_chi1_mean, *p_chi1_distb);
     /******************************************************/
      // p_chi1->Print(); 
      // p_chi1_mean->Print();
      // p_chi1_distb->Print();
     /*******************************************************/

     // ߂ p_chi3 ̓ςƂ, p_chi3_mean Ɋi[.
     zonal_average(*p_chi3, *p_chi3_mean);
     // p_chi3, p_chi3_mean gďvZ, p_chi1_distb Ɋi[.
     calc_disturb(*p_chi3, *p_chi3_mean, *p_chi3_distb);
     /*************************************************************/
     // p_chi3->Print(); 
     // p_chi3_mean->Print(); 
     // p_chi3_distb->Print();
     /*************************************************************/
     
     // Ϗɑ΂ĉ
     // 1 ^CXebvڂ͖CA t ݂̂̒lg.
     calc_PV_Mean_Time_Evol(pm, dm);
     
     // ɑ΂ĉ
     if(distb_cal_flag){
        calc_PV_Disturb_Time_Evol(pm, dm);
     }

     // ϏƏ, t+1 bɂ|eVQxƂ
     Add_Mean_Disturb( *dm.Get_PV_Mean(Field_Data_Manager::PV_MEAN_LAYER1), 
                       *dm.Get_PV_Disturb(Field_Data_Manager::PV_DISTB_LAYER1),
		       *dm.Get_PV(Field_Data_Manager::PV_LAYER1_AFTER) );

     Add_Mean_Disturb( *dm.Get_PV_Mean(Field_Data_Manager::PV_MEAN_LAYER3), 
                       *dm.Get_PV_Disturb(Field_Data_Manager::PV_DISTB_LAYER3),
		       *dm.Get_PV(Field_Data_Manager::PV_LAYER3_AFTER) );

     if( first_run_flag ){
         // first_run ȍ~, leap frog @ĝ,Ƃ̌W̒lɖ߂.
	 alpha = alpha * 2.0;
	 c = c * 2.0;
	 h = h * 2.0;
	 pm.k = pm.k * 2.0;
         mu3 = 1.0 + 1.5 * pm.k * pm.del_t;

	 first_run_flag = false;
     }
}

void Forecast::Diagnose_Stream_Function(Param_List& pm, Field_Data_Manager& dm)
{
    //  t+1 ɂ|eVQx̕Ϗp, ֐̕Ϗ߂.
    // ډ@ɂĘAꎟ.
    diagnose_SF_Mean(pm, dm);
    
    //  t+1 ɂ|eVQx̏p, ֐̏߂.
    // ɘa@ɂĘAꎟ.
    if(distb_cal_flag){
        diagnose_SF_Disturb(pm, dm);
     }

     // ϏƏ, t+1 bɂ闬֐Ƃ
     Add_Mean_Disturb( *dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER1_AFTER), 
                       *dm.Get_Stream_Func_Disturb(Field_Data_Manager::SF_DISTB_LAYER1_AFTER),
		       *dm.Get_Stream_Func(Field_Data_Manager::SF_LAYER1_AFTER) );

     Add_Mean_Disturb( *dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER3_AFTER), 
                       *dm.Get_Stream_Func_Disturb(Field_Data_Manager::SF_DISTB_LAYER3_AFTER),
		       *dm.Get_Stream_Func(Field_Data_Manager::SF_LAYER3_AFTER) );
}


void Forecast::calc_PV_Time_Evol_Prepair(Param_List& pm, Field_Data_Manager& dm)
{
     // 250, 750 hPa ̃|eVQx̎ԔW̎̉E chi1, chi3 vZ.
     GMatrix* p_pv1_a = dm.Get_PV(Field_Data_Manager::PV_LAYER1_AFTER);   
     GMatrix* p_pv1_b = dm.Get_PV(Field_Data_Manager::PV_LAYER1_BEFORE);   
     GMatrix* p_pv1_b2 = dm.Get_PV(Field_Data_Manager::PV_LAYER1_BEFORE2);

     GMatrix* p_pv3_a = dm.Get_PV(Field_Data_Manager::PV_LAYER3_AFTER);   
     GMatrix* p_pv3_b = dm.Get_PV(Field_Data_Manager::PV_LAYER3_BEFORE);   
     GMatrix* p_pv3_b2 = dm.Get_PV(Field_Data_Manager::PV_LAYER3_BEFORE2);

     GMatrix* p_sf1 = dm.Get_Stream_Func(Field_Data_Manager::SF_LAYER1_BEFORE);
     GMatrix* p_sf3 = dm.Get_Stream_Func(Field_Data_Manager::SF_LAYER3_BEFORE);
     
     double tmp;
     double rad_h;
     double lambda = pm.vertical_stability * pm.del_x * pm.del_x;
     double ek_pv3_cof;
     double Jmx = pm.num_y - 1;

     if( first_run_flag ){
         // first run ̏ꍇ͑OiIC[@ɂȂ悤ɒ. 
	 std::cout << "first run config" << std::endl;
         ek_pv3_cof = 3.0;
	 p_pv1_b2 = p_pv1_b;
	 p_pv3_b2 = p_pv3_b;
     }else{
         ek_pv3_cof = 1.5;
     }

     // 炩߈ڗŃRrAɓnsIuWFNgvZĂ.
     for(int i= 0; i < pm.num_x; i++){
          for(int j=0; j < pm.num_y; j++){
	         p_adv_mx1->Set(i, j, p_pv1_b->Get(i, j) + b * j);
	         p_adv_mx3->Set(i, j, p_pv3_b->Get(i, j) + b * j);
          }
     }
     
     
     // 250,750 hPa ̃|eVQx̎ԔW̎̉E chi1, chi3 ]
     for(int i= 0; i < pm.num_x; i++){
          for(int j=0; j < pm.num_y; j++){

	      rad_h =  h * ( 2 * j - Jmx ) / Jmx;  // ˉM̍
	      
	      // 250 hpa
              tmp = (*p_pv1_b2)(i, j)                          //  t-1 ɂ|eVQx
 	           + c * J(pm, *p_adv_mx1, *p_sf1, i, j)       // ڗ̌vZ 
 	           + alpha * L(pm, *p_pv1_b2, i, j)            // QgU̍
		   + rad_h;                                    // ˉM̍
  	      
	      p_chi1->Set(i, j, tmp); 
	      
	      // 750 hpa
              tmp = (*p_pv3_b2)(i, j)                          //  t-1 ɂ|eVQx
 	           + c * J(pm, *p_adv_mx3, *p_sf3, i, j)       // ڗ̌vZ 
 	           + alpha * L(pm, *p_pv3_b2, i, j)            // QgU̍
		   - rad_h                                     // ˉM̍
  	           - pm.k * pm.del_t *                            // GN}psOɊ֌W鍀
		     ( 
		      ek_pv3_cof * (*p_pv3_b2)(i, j) - (*p_pv1_b)(i, j) 
		      - 4.0 * lambda * ( (*p_sf1)(i, j) - (*p_sf3)(i, j))
		     );

	      p_chi3->Set(i, j, tmp); 
	 }
     }
     
}


void Forecast::calc_PV_Mean_Time_Evol(Param_List& pm, Field_Data_Manager& dm)
{
     // vZĂϏɑ΂|eVQx̉Eӂ̒l chi1_mean, chi3_mean p,
     // del_t b̃|eVQxvZ.
     // Ȃ, kEɂ 250, 750 hPa ̗֐̕Ϗ͔CӂȂ̂ŌŒ肷.
 
     // 250 hPa ̓[Eɂ闬֐̓ς擾.
     GVector* p_pv1_m = dm.Get_PV_Mean(Field_Data_Manager::PV_MEAN_LAYER1);

     // 750 hPa ̓[Eɂ闬֐̓ς擾.
     GVector* p_pv3_m = dm.Get_PV_Mean(Field_Data_Manager::PV_MEAN_LAYER3);
     
     /************************************************/
      //p_pv1_m->Print();@p_pv3_m->Print();
     /***********************************************/
     
     solve_ODE_Direct(pm, p_A, p_pv1_m, p_chi1_mean, - alpha * epsilon * epsilon, mu1);
     solve_ODE_Direct(pm, p_A, p_pv3_m, p_chi3_mean, - alpha * epsilon * epsilon, mu3);
     
} 

void Forecast::calc_PV_Disturb_Time_Evol(Param_List& pm, Field_Data_Manager& dm)
{
     // vZĂɑ΂|eVQx̉Eӂ̒l chi1_diturb, chi3_disturb p,
     // del_t b̃|eVQxvZ.
     // ̓Iɂ,ĂłAꎟɘa@(KEXEUCf@)łƂ.
     
     GMatrix* p_pv1_d = dm.Get_PV_Disturb(Field_Data_Manager::PV_DISTB_LAYER1);   
     GMatrix* p_pv3_d = dm.Get_PV_Disturb(Field_Data_Manager::PV_DISTB_LAYER3);   

     int imax = p_pv1_d->Row();
     int jmax = p_pv1_d->Column();
     double tmp;
     
     // kȄ̓Kp
     // (ǂɂă|eVQx̏͏ 0 )
     for(int i=0; i < imax; i++){
         (*p_pv1_d)(i, 0) = 0.0;
	 (*p_pv1_d)(i, jmax-1) = 0.0;
         
	 (*p_pv3_d)(i, 0) = 0.0;
	 (*p_pv3_d)(i, jmax-1) = 0.0;
     }

     // ɘa@ŗpŏ̒l( first guess ) t ̒lgĂ߂Ă
     // <pv_distb>s_ij = chi_distb_s_ij / mu_s
     for(int i=0; i < imax; i++){
         for(int j = 1; j < jmax-1; j++){
	     (*p_pv1_d)(i, j) = (*p_chi1_distb)(i, j) / mu1;
	     (*p_pv3_d)(i, j) = (*p_chi3_distb)(i, j) / mu3;
	 }
     }

     // 臒l 600 [ m^2/s] Ƃ, 250 hPa, 750 hPa ɑ΂Aɘa@ŉ.
     solve_Poisson_Eq_Relax(pm, p_pv1_d, p_chi1_distb, - alpha, mu1, 600.0);
     solve_Poisson_Eq_Relax(pm, p_pv3_d, p_chi3_distb, - alpha, mu3, 600.0);
     
}

void Forecast::diagnose_SF_Mean(Param_List& pm, Field_Data_Manager& dm)
{

    GVector *p_sf_br = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_BAROTOPIC_MODE);
    GVector *p_sf_cl = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_BAROCLINIC_MODE);
    GVector *p_sf1 = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER1_AFTER);
    GVector *p_sf3 = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER3_AFTER);
    GVector *p_pv1 = dm.Get_PV_Mean(Field_Data_Manager::PV_MEAN_LAYER1);
    GVector *p_pv3 = dm.Get_PV_Mean(Field_Data_Manager::PV_MEAN_LAYER3);
    double tmp;
    double ep2 = epsilon * epsilon;
    double jmax = p_sf3->Size();

    // X[hɑ΂A.
    p_pv_mean13->Sub(*p_pv1, *p_pv3);
    solve_ODE_Direct(pm, p_A, p_sf_cl, p_pv_mean13, 
                     ep2, - 2.0 * pm.vertical_stability * pm.del_x * pm.del_x);

    // [hɑ΂Aɂ, p_sf3[0] ^邱Ƃ, QI p_sf1, p_sf3 肷.
    tmp = 2.0 * (*p_sf3)(0) + (*p_sf_cl)(0);
    p_sf_br->Set(0, tmp);
    p_sf_br->Set(1, tmp);
    p_pv_mean13->Add(*p_pv1, *p_pv3);

    for(int j=1; j < jmax - 2; j++){
        tmp =  2.0 * p_sf_br->Get(j) - p_sf_br->Get(j-1) 
	      + p_pv_mean13->Get(j) / ep2;
	p_sf_br->Set(j+1, tmp);
    }

    p_sf_br->Set(jmax-1, tmp);

    // p_sf1, p_sf3 o.
    p_sf1->Add(*p_sf_br, *p_sf_cl);
    p_sf1->Scalar_Product(0.5);

    p_sf3->Sub(*p_sf_br, *p_sf_cl);
    p_sf3->Scalar_Product(0.5);
}

void Forecast::diagnose_SF_Disturb(Param_List& pm, Field_Data_Manager& dm)
{

    GMatrix *p_sf1_a = dm.Get_Stream_Func_Disturb(Field_Data_Manager::SF_DISTB_LAYER1_AFTER);
    GMatrix *p_sf3_a = dm.Get_Stream_Func_Disturb(Field_Data_Manager::SF_DISTB_LAYER3_AFTER);
    GMatrix *p_sf1_b = dm.Get_Stream_Func_Disturb(Field_Data_Manager::SF_DISTB_LAYER1_BEFORE);
    GMatrix *p_sf3_b = dm.Get_Stream_Func_Disturb(Field_Data_Manager::SF_DISTB_LAYER3_BEFORE);
    GMatrix *p_sf1_b2 = dm.Get_Stream_Func_Disturb(Field_Data_Manager::SF_DISTB_LAYER1_BEFORE2);
    GMatrix *p_sf3_b2 = dm.Get_Stream_Func_Disturb(Field_Data_Manager::SF_DISTB_LAYER3_BEFORE2);
    GMatrix *p_pv1_d = dm.Get_PV_Disturb(Field_Data_Manager::PV_DISTB_LAYER1);
    GMatrix *p_pv3_d = dm.Get_PV_Disturb(Field_Data_Manager::PV_DISTB_LAYER3);
    double tmp;
    int imax = p_pv1_d->Row();
    int jmax = p_pv1_d->Column();

     // kȄ̓Kp
     // (ǂɂė֐̏͏ 0 )
     for(int i=0; i < imax; i++){
	 (*p_sf1_a)(i, 0) = 0.0;
	 (*p_sf1_a)(i, jmax-1) = 0.0;
	 
	 (*p_sf3_a)(i, 0) = 0.0;
	 (*p_sf3_a)(i, jmax-1) = 0.0;
     }

     // ɘa@ŗpŏ̒l( first guess ) t, t-1 ̒lɂ`⊮,߂Ă.
     // <sf_distb>s_ij = 2.0 * sf_distb_t - sf_distb_t-1 
     //
     for(int i=0; i < imax; i++){
         for(int j = 1; j < jmax-1; j++){
	     
	     (*p_sf1_a)(i, j) = 2.0 * (*p_sf1_b)(i, j) - (*p_sf1_b2)(i, j);
	     (*p_sf3_a)(i, j) = 2.0 * (*p_sf3_b)(i, j) - (*p_sf3_b2)(i, j);
	 }
     }

     // 臒l 3.0e05 / 16.0 [ m^2/s] Ƃ, 250 hPa, 750 hPa ɑ΂Aɘa@ŉ.
     solve_Poisson_Eq_Relax2(pm, p_sf1_a, p_sf3_a, p_pv1_d, p_pv3_d, 1.0, pm.vertical_stability * pm.del_x * pm.del_x, 3.0e05 / 8.0, 1.25);
     
}


void Forecast::Diagnose_v_p_Velocity(Param_List& pm, Field_Data_Manager& dm)
{
    GMatrix* p_omega = dm.Get_Omega();
    GMatrix* p_sf1_a = dm.Get_Stream_Func(Field_Data_Manager::SF_LAYER1_AFTER);
    GMatrix* p_sf1_b = dm.Get_Stream_Func(Field_Data_Manager::SF_LAYER1_BEFORE);
    GMatrix* p_sf3_a = dm.Get_Stream_Func(Field_Data_Manager::SF_LAYER3_AFTER);
    GMatrix* p_sf3_b = dm.Get_Stream_Func(Field_Data_Manager::SF_LAYER3_BEFORE);

    double tmp;
    double conf = pm.vertical_stability * pm.p2 / pm.f0;
    int imax = p_omega->Row();
    int jmax = p_omega->Column();
    int Jmx = jmax-1;
    double del_sf1;
    double del_sf3;
    
    GMatrix* p_sf1_tm;
    GMatrix* p_sf3_tm;

    p_sf1_tm = new GMatrix(imax, jmax);
    p_sf3_tm = new GMatrix(imax, jmax);

    // after, before ̗֐̃f[^gĎԕςƂ.
    p_sf1_tm->Add(*p_sf1_a, *p_sf1_b);
    p_sf1_tm->Scalar_Product(0.5);
    p_sf3_tm->Add(*p_sf3_a, *p_sf3_b);
    p_sf3_tm->Scalar_Product(0.5);
    
    // ꂼ̊iq_,  p xvZ.
    for(int i=0; i < imax; i++){
        for(int j=0; j < jmax; j++){
	    
	    del_sf1 = (*p_sf1_a)(i, j) - (*p_sf1_b)(i, j);
	    del_sf3 = (*p_sf3_a)(i, j) - (*p_sf3_b)(i, j);
            
            tmp = conf * 
	         (
	            ( del_sf1 - del_sf3 ) / pm.del_t 
		    - J(pm, *p_sf1_tm, *p_sf3_tm, i, j) / ( 4.0 * pm.del_x * pm.del_y )
		    + 2.0 * Earth::R_dry * pm.H * ( 2.0 * j - Jmx ) / ( pm.f0 * Earth::Cp_dry * Jmx )
		    - pm.A * ( L(pm, *p_sf1_tm, i, j) - L(pm, *p_sf3_tm, i, j) ) / ( pm.del_x * pm.del_x )
		 );

            p_omega->Set(i, j, tmp);
	}
    }

    delete p_sf1_tm;
    delete p_sf3_tm;

    // vZ p x̏ꂩ瓌ϏƏ߂
    GVector* p_omega_mean = dm.Get_Omega_Mean();
    GMatrix* p_omega_distb = dm.Get_Omega_Disturb();

    zonal_average(*p_omega, *p_omega_mean);   
    calc_disturb(*p_omega, *p_omega_mean, *p_omega_distb);
}


void Forecast::Diagnose_Vorcity(Param_List& pm, Field_Data_Manager& dm)
{
    GMatrix* p_vr1 = dm.Get_Vorcity(Field_Data_Manager::VR_LAYER1);
    GMatrix* p_vr3 = dm.Get_Vorcity(Field_Data_Manager::VR_LAYER3);
    
    GMatrix* p_sf1_a = dm.Get_Stream_Func(Field_Data_Manager::SF_LAYER1_AFTER);
    GMatrix* p_sf1_b = dm.Get_Stream_Func(Field_Data_Manager::SF_LAYER1_BEFORE);
    GMatrix* p_sf3_a = dm.Get_Stream_Func(Field_Data_Manager::SF_LAYER3_AFTER);
    GMatrix* p_sf3_b = dm.Get_Stream_Func(Field_Data_Manager::SF_LAYER3_BEFORE);
    
    int imax = pm.num_x;
    int jmax = pm.num_y;
    double tmp;
    double delx2 = pm.del_x * pm.del_x;

    GMatrix* p_sf1_tm;
    GMatrix* p_sf3_tm;

    p_sf1_tm = new GMatrix(imax, jmax);
    p_sf3_tm = new GMatrix(imax, jmax);

    // after, before ̗֐̃f[^gĎԕςƂ.
    p_sf1_tm->Add(*p_sf1_a, *p_sf1_b);
    p_sf1_tm->Scalar_Product(0.5);
    p_sf3_tm->Add(*p_sf3_a, *p_sf3_b);
    p_sf3_tm->Scalar_Product(0.5);
    
    for(int i=0; i < imax; i++){
        for(int j=0; j < jmax; j++){

	    (*p_vr1)(i, j) = L(pm, *p_sf1_tm, i, j) / delx2;
	    (*p_vr3)(i, j) = L(pm, *p_sf3_tm, i, j) / delx2;
	}
    }

    delete p_sf1_tm;
    delete p_sf3_tm;

    GVector* p_vr1_mean = dm.Get_Vorcity_Mean(Field_Data_Manager::VR_LAYER1);
    GVector* p_vr3_mean = dm.Get_Vorcity_Mean(Field_Data_Manager::VR_LAYER3);
    GMatrix* p_vr1_distb = dm.Get_Vorcity_Disturb(Field_Data_Manager::VR_LAYER1);
    GMatrix* p_vr3_distb = dm.Get_Vorcity_Disturb(Field_Data_Manager::VR_LAYER3);

    zonal_average(*p_vr1, *p_vr1_mean);
    calc_disturb(*p_vr1, *p_vr1_mean, *p_vr1_distb);

    zonal_average(*p_vr3, *p_vr3_mean);
    calc_disturb(*p_vr3, *p_vr3_mean, *p_vr3_distb);
}


void Forecast::Create_Disturb_Field(Param_List& pm, Field_Data_Manager& dm)
{
    GMatrix* p_sf1_d = dm.Get_Stream_Func_Disturb(Field_Data_Manager::SF_DISTB_LAYER1_AFTER);
    GMatrix* p_sf3_d = dm.Get_Stream_Func_Disturb(Field_Data_Manager::SF_DISTB_LAYER3_AFTER);
 
    int imax = pm.num_x;
    int J = pm.num_y - 1;
    double tmp, tmp1, tmp2, tmp3, tmp4;

    p_sf1_d->Fill(0.0);
    p_sf3_d->Fill(0.0);
 
    // ^邽߂̃V[h, g.
    srand((unsigned) time(NULL));
    
    // 0  100 ܂ł̗ЂƂ܂֐̏p
    // sIuWFNgɊi[.
    double zonal_ave;

    for(int j=1; j < J; j++){
        zonal_ave = 0.0;

	for(int i=0; i < imax; i++){
	    tmp = rand() % 100;

	    p_sf1_d->Set(i, j, tmp);
	    p_sf3_d->Set(i, j, tmp);
	    
	    zonal_ave += tmp;
	}

        // ς֐̐ۓ, 0 ɂȂ悤ɒ߂.
	zonal_ave = zonal_ave / imax;
        for(int i=0; i < imax; i++){
	    p_sf1_d->Set(i, j, (*p_sf1_d)(i, j) - zonal_ave);
	    p_sf3_d->Set(i, j, (*p_sf3_d)(i, j) - zonal_ave);
	}
    }
    
    // ̉^GlM[, 768 units ɂȂ悤ɐU𒲐߂. 
    int iup;
    double ep2 = epsilon * epsilon;
    double K_d;
    double conf;
 
    tmp = 0;
    for(int i=0; i < imax; i++){
       for(int j=1; j < J; j++){
           if(i == imax-1){ 
	      iup = 0;
           }else{
	      iup = i + 1;
	   }

           tmp1 = (*p_sf1_d)(iup, j) - (*p_sf1_d)(i, j); 
           tmp2 = (*p_sf3_d)(iup, j) - (*p_sf3_d)(i, j); 
           tmp3 = (*p_sf1_d)(i, j+1) - (*p_sf1_d)(i, j); 
           tmp4 = (*p_sf3_d)(i, j+1) - (*p_sf3_d)(i, j); 

	   tmp += ( tmp1 * tmp1 + tmp2 * tmp2 ) + ep2 * ( tmp3 * tmp3 + tmp4 * tmp4 );
       }
    }

    K_d = 10.0 / ( 2.0 * J * imax * pm.del_x * pm.del_x ) * tmp;
    conf = 768.0 / K_d; //768.0 / K_d;

    p_sf1_d->Scalar_Product( std::sqrt(conf) );
    p_sf3_d->Scalar_Product( std::sqrt(conf) );
   
    
    // 쐬֐̏𕽋ϏƉ.
    // Oɏ}Õf[^ After  Before ɈڂĂ(GlM[̕]ɕKv)
    dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER1_BEFORE)->Copy(*dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER1_AFTER));
    dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER3_BEFORE)->Copy(*dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER3_AFTER));
    dm.Get_Stream_Func_Disturb(Field_Data_Manager::SF_DISTB_LAYER1_BEFORE)->Copy(*p_sf1_d);
    dm.Get_Stream_Func_Disturb(Field_Data_Manager::SF_DISTB_LAYER3_BEFORE)->Copy(*p_sf3_d);
    
    Add_Mean_Disturb(*dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER1_AFTER),
                     *p_sf1_d,
                     *dm.Get_Stream_Func(Field_Data_Manager::SF_LAYER1_AFTER));
    
    Add_Mean_Disturb(*dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER3_AFTER),
                     *p_sf3_d,
                     *dm.Get_Stream_Func(Field_Data_Manager::SF_LAYER3_AFTER));


    // 쐬֐̏ꂩ, }̉QxщxvZĂ. 
    // GlM[TCN̕]ɕKvƂȂ. 
    Diagnose_v_p_Velocity(pm, dm);
    
    dm.Get_Stream_Func(Field_Data_Manager::SF_LAYER1_BEFORE)->Copy(*dm.Get_Stream_Func(Field_Data_Manager::SF_LAYER1_AFTER));
    dm.Get_Stream_Func(Field_Data_Manager::SF_LAYER3_BEFORE)->Copy(*dm.Get_Stream_Func(Field_Data_Manager::SF_LAYER3_AFTER));
    Diagnose_Vorcity(pm, dm);
    

    // 쐬֐̏ꂩ,|eVQx̏vZ.ĕϏƉ.
    GMatrix* p_pv1_d = dm.Get_PV_Disturb(Field_Data_Manager::PV_DISTB_LAYER1);
    GMatrix* p_pv3_d = dm.Get_PV_Disturb(Field_Data_Manager::PV_DISTB_LAYER3);
    
    for(int i=0; i < imax; i++){
        for(int j=1; j < J; j++){
	   tmp = L(pm, *p_sf1_d, i, j) - pm.vertical_stability * ( (*p_sf1_d)(i, j) - (*p_sf3_d)(i, j) );
	   p_pv1_d->Set(i, j, tmp);

	   tmp = L(pm, *p_sf3_d, i, j) + pm.vertical_stability * ( (*p_sf1_d)(i, j) - (*p_sf3_d)(i, j) );
	   p_pv3_d->Set(i, j, tmp);
	}
    }

    Add_Mean_Disturb(*dm.Get_PV_Mean(Field_Data_Manager::PV_MEAN_LAYER1),
                     *p_pv1_d,
                     *dm.Get_PV(Field_Data_Manager::PV_LAYER1_AFTER));
    
    Add_Mean_Disturb(*dm.Get_PV_Mean(Field_Data_Manager::PV_MEAN_LAYER3),
                     *p_pv3_d,
                     *dm.Get_PV(Field_Data_Manager::PV_LAYER3_AFTER));
 
    /*
    Add_Mean_Disturb(*dm.Get_PV_Mean(Field_Data_Manager::PV_MEAN_LAYER1),
                     *p_pv1_d,
                     *dm.Get_PV(Field_Data_Manager::PV_LAYER1_BEFORE));
    
    Add_Mean_Disturb(*dm.Get_PV_Mean(Field_Data_Manager::PV_MEAN_LAYER3),
                     *p_pv3_d,
                     *dm.Get_PV(Field_Data_Manager::PV_LAYER3_BEFORE));
    */
}
