#ifndef FORECAST_UTIL_H
#define FORECAST_UTIL_H

#include "Param_List.h"
#include "Field_Data_Manager.h"

// C++ 標準クラスライブラリのインクルード
#include <iostream>
#include <cmath>

// GMathLib を利用するためのヘッダーの読み込み
#include <gmathcore/GMatrix.h>
#include <gmathcore/GVector.h>
#include <gmathcore/GLinear_Homo_Eq.h>

using namespace GMathLib;

/**
 * 予報クラスで用いる汎用関数をまとめておく名前空間
 */
namespace Forecast_Util{

/*
 * ラプラシアン の差分式の定義
 */
double L(Param_List& pm, GMatrix& mx, int i, int j);


/*
 * ヤコビアン の差分式の定義
 */   
double J(Param_List& pm, GMathLib::GMatrix& mx1, GMathLib::GMatrix& mx2, int i, int j);


/*
 * 常微分方程式の連立一次方程式を直接的解法で解く. 
 */
void solve_ODE_Direct(Param_List& pm, GMatrix* p_A, GVector* p_mean_mx, GVector* p_eval_mx, double A, double B);

/*
 * 擾乱のポテンシャル渦度方程式を緩和法で解く関数.
 * 引数の v は, 過緩和係数である.(デフォルトは, 1.0 であり, その場合はガウス・ザイデル法となる)
 */
void solve_Poisson_Eq_Relax(Param_List& pm, GMatrix* p_V, GMatrix* p_S, 
                            double A, double B, double conv_value, double v=1.0);

/*
 * 擾乱の流線関数場の Poisson (ホルムヘルツ) 方程式を緩和法で解く関数.
 * 引数の v は, 過緩和係数である.(デフォルトは, 1.0 であり, その場合はガウス・ザイデル法となる)
 */
void solve_Poisson_Eq_Relax2(Param_List& pm, GMatrix* p_SF1, GMatrix* p_SF3, 
                             GMatrix* p_PV1, GMatrix* p_PV3, double A, double B, double conv_value, double v);

/*
 * もとの行列データの東西平均を取る関数
 */
void zonal_average(GMatrix& src_mx, GVector& avg_mx);

/*
 * もとの行列データと東西平均場の行列データの差をとって
 * 擾乱場の行列データを求める関数.
 */
void calc_disturb(GMatrix& src_mx, GVector& mean_mx, GMatrix& distb_mx);

/*
 * 平均場の行列データと擾乱場の行列データを加えて, もとの行列データを作成する.
 */
void Add_Mean_Disturb(GVector& mean_mx, GMatrix& distb_mx, GMatrix& store_mx);

}

#endif
